import {Component, Input, OnChanges, OnInit, SimpleChanges, ViewChild} from '@angular/core';
import {FormControl} from "@angular/forms";
import {ChangeContext, LabelType, Options} from "ngx-slider-v2";
import {ApexAxisChartSeries, ChartComponent, ChartType} from "ng-apexcharts";
import {HistoryService, TaskgroupActivityInfo, TaskgroupPathInfo, TaskgroupService} from "../../../../api";
import * as moment from "moment/moment";
import {ChartOptions} from "../taskgroup-activity.component";

@Component({
  selector: 'app-simple-activity-diagram',
  templateUrl: './simple-activity-diagram.component.html',
  styleUrls: ['./simple-activity-diagram.component.css']
})
export class SimpleActivityDiagramComponent implements OnChanges {
  @Input('selectedChartype') selectedChartype: string | undefined
  @Input() selectedTaskgroupPath: TaskgroupPathInfo | undefined;

  sliderControl: FormControl = new FormControl()
  dateRange: Date[] = this.createDateRange();
  selectedDateRange: Date[] = this.dateRange;
  value: number = this.dateRange[0].getTime();
  options: Options = {
    stepsArray: this.dateRange.map((date: Date) => {
      return { value: date.getTime() };
    }),
    translate: (value: number, label: LabelType): string => {
      return new Date(value).toDateString();
    },
    floor: 0,
    ceil: this.dateRange.length,
  };

  @ViewChild("chart") chart?: ChartComponent;

  chartOptions: Partial<ChartOptions> | undefined;

  constructor(private taskgroupService: TaskgroupService,
              private historyService: HistoryService) {
  }
  ngOnChanges() {
    this.fetchData()
  }

  fetchData() {
    const startingDate = moment(this.dateRange[0]).format("yyyy-MM-DD");
    const endingDate = moment(this.dateRange[this.dateRange.length-1]).format("yyyy-MM-DD");

    if(this.selectedTaskgroupPath == undefined) {
      this.historyService.statisticsStartingDateEndingDateGet(startingDate, endingDate).subscribe({
        next: resp => {
          const series = this.createChartSeries(resp);
          this.chartOptions = this.createChartOptions(series);
        }
      })
    } else {
      this.historyService.statisticsTaskgroupIDStartingDateEndingDateGet(
        this.selectedTaskgroupPath.rootTasktroup.taskgroupID,
        startingDate,
        endingDate
      ).subscribe({
        next: resp => {
          const series = this.createChartSeries(resp);
          this.chartOptions = this.createChartOptions(series)
        }
      })
    }
  }

  createChartSeries(data: TaskgroupActivityInfo[]) {
    const series: ApexAxisChartSeries = [];
    data.forEach(taskgroupActivityInfo => {
      const data: any[] = [];
      this.dateRange.map(date => {
        const selectedActivity = taskgroupActivityInfo.activityInfos.find(activity => moment(date).isSame(moment(activity.date), "day"))
        if(selectedActivity != undefined) {
          data.push(selectedActivity.activeMinutes)
        } else {
          data.push(0)
        }
      })

      series.push({
        name: taskgroupActivityInfo.taskgroup.taskgroupName,
        data: data
      })
    })

    return series;
  }

  createChartOptions(series: any[] = []) {
    return {
      series: series,
      chart: {
        height: 350,
        type: this.selectedChartype as ChartType,
        stacked: true
      },
      title: {
        text: ""
      },
      xaxis: {
        categories: this.selectedDateRange.map(date => date.toDateString())
      }
    }
  }

  createDateRange(): Date[] {
    const dates: Date[] = [];
    for (let i: number = 1; i <= 30; i++) {

      dates.push(moment().subtract(30-i, 'd').toDate());
    }
    this.sliderControl.setValue([dates[0], dates[dates.length-1]])
    return dates;
  }

  createDateRangeBetween(minValue: moment.Moment, maxValue: moment.Moment) {
    const dates: Date[] = [];
    let currentDate = moment(minValue);

    while(currentDate <= maxValue) {
      dates.push(currentDate.toDate());
      currentDate = currentDate.add(1, 'days');
    }

    return dates;
  }

  onUserChange(changeContext: ChangeContext) {
    console.log("min " + moment(changeContext.value).format("YYYY-MM-DD"));
    console.log("max " + moment(changeContext.highValue!).format("YYYY-MM-DD"))
    this.selectedDateRange = this.createDateRangeBetween(moment(changeContext.value), moment(changeContext.highValue!))
  }

  setSelectedChartType(selectedChartype: string) {
    //this.selectedChartype = selectedChartype;

  }
}
